﻿#!/usr/bin/env python

import os
import sys
from rare import rare_marker

NO_E = -1.0
train_file = 'gene.train'
rare_train_file = 'gene.1.train'
count_file = 'gene.1.counts'
dev_file = 'gene.dev'
tag_set = set(['*', 'STOP'])
e_d = {}


def get_rare(word = None):
    """
    word - not used
    """
    return "_RARE_"


def generate_e_dict(count_file):
    """
    Generates emission parameters dictionary.
    Returns the dictionary.
    """
    e_dict = {}
    tag_dict = {}
    with open(count_file, "r") as fin:
        for l in fin:
            line = l.strip()
            if line:
                fields = line.split(" ")
                if fields[1] == "WORDTAG":
                    key = fields[3], fields[2]
                    e_dict[key] = float(fields[0])
                elif fields[1] == "1-GRAM":
                    tag_set.add(fields[2])
                    tag_dict[fields[2]] = float(fields[0])
    for aKey in e_dict.keys():
        e_dict[aKey] = e_dict[aKey] / tag_dict[aKey[1]]
    return e_dict


def get_e(word, tag):
    global e_d
    if len(e_d) == 0:
        return NO_E
    search_key = word, tag
    if search_key in e_d:
        return e_d[search_key]
    else:
        return NO_E


def search_e_dict(word):
    e = NO_E
    max_tag = None
    for tag in tag_set:
        this_e = get_e(word, tag)
        if this_e > e:
            e = this_e
            max_tag = tag
    return max_tag


def tag_it():
    with open(dev_file) as fdev:
        for l in fdev:
            line = l.strip()
            if len(line) == 0:
                print
            else:
                tag = search_e_dict(line)
                if tag == None:
                    tag = search_e_dict(get_rare())
                if tag != None:
                    print line, tag


if __name__ == "__main__":
    # Usage:
    #     python part1.py [dev_file]
    if len(sys.argv) >= 2:
        dev_file = sys.argv[1]
    
    rm = rare_marker(get_rare, train_file)
    with open(rare_train_file, 'w') as f_out:
        rm.get_rare_train(f_out)

    os.system('python count_freqs.py ' + rare_train_file + ' > ' + count_file)

    e_d = generate_e_dict(count_file)
    tag_it()
